diff src/events.h @ 853:2b6fa2618f76

[xemacs-hg @ 2002-05-28 08:44:22 by ben] merge my stderr-proc ws make-docfile.c: Fix places where we forget to check for EOF. code-init.el: Don't use CRLF conversion by default on process output. CMD.EXE and friends work both ways but Cygwin programs don't like the CRs. code-process.el, multicast.el, process.el: Removed. Improvements to call-process-internal: -- allows a buffer to be specified for input and stderr output -- use it on all systems -- implement C-g as documented -- clean up and comment call-process-region uses new call-process facilities; no temp file. remove duplicate funs in process.el. comment exactly how coding systems work and fix various problems. open-multicast-group now does similar coding-system frobbing to open-network-stream. dumped-lisp.el, faces.el, msw-faces.el: Fix some hidden errors due to code not being defined at the right time. xemacs.mak: Add -DSTRICT. ================================================================ ALLOW SEPARATION OF STDOUT AND STDERR IN PROCESSES ================================================================ Standard output and standard error can be processed separately in a process. Each can have its own buffer, its own mark in that buffer, and its filter function. You can specify a separate buffer for stderr in `start-process' to get things started, or use the new primitives: set-process-stderr-buffer process-stderr-buffer process-stderr-mark set-process-stderr-filter process-stderr-filter Also, process-send-region takes a 4th optional arg, a buffer. Currently always uses a pipe() under Unix to read the error output. (#### Would a PTY be better?) sysdep.h, sysproc.h, unexfreebsd.c, unexsunos4.c, nt.c, emacs.c, callproc.c, symsinit.h, sysdep.c, Makefile.in.in, process-unix.c: Delete callproc.c. Move child_setup() to process-unix.c. wait_for_termination() now only needed on a few really old systems. console-msw.h, event-Xt.c, event-msw.c, event-stream.c, event-tty.c, event-unixoid.c, events.h, process-nt.c, process-unix.c, process.c, process.h, procimpl.h: Rewrite the process methods to handle a separate channel for error input. Create Lstreams for reading in the error channel. Many process methods need change. In general the changes are fairly clear as they involve duplicating what's used for reading the normal stdout and changing for stderr -- although tedious, as such changes are required throughout the entire process code. Rewrote the code that reads process output to do two loops, one for stdout and one for stderr. gpmevent.c, tooltalk.c: set_process_filter takes an argument for stderr. ================================================================ NEW ERROR-TRAPPING MECHANISM ================================================================ Totally rewrite error trapping code to be unified and support more features. Basic function is call_trapping_problems(), which lets you specify, by means of flags, what sorts of problems you want trapped. these can include -- quit -- errors -- throws past the function -- creation of "display objects" (e.g. buffers) -- deletion of already-existing "display objects" (e.g. buffers) -- modification of already-existing buffers -- entering the debugger -- gc -- errors->warnings (ala suspended errors) etc. All other error funs rewritten in terms of this one. Various older mechanisms removed or rewritten. window.c, insdel.c, console.c, buffer.c, device.c, frame.c: When creating a display object, added call to note_object_created(), for use with trapping_problems mechanism. When deleting, call check_allowed_operation() and note_object deleted(). The trapping-problems code records the objects created since the call-trapping-problems began. Those objects can be deleted, but none others (i.e. previously existing ones). bytecode.c, cmdloop.c: internal_catch takes another arg. eval.c: Add long comments describing the "five lists" used to maintain state (backtrace, gcpro, specbind, etc.) in the Lisp engine. backtrace.h, eval.c: Implement trapping-problems mechanism, eliminate old mechanisms or redo in terms of new one. frame.c, gutter.c: Flush out the concept of "critical display section", defined by the in_display() var. Use an internal_bind() to get it reset, rather than just doing it at end, because there may be a non-local exit. event-msw.c, event-stream.c, console-msw.h, device.c, dialog-msw.c, frame.c, frame.h, intl.c, toolbar.c, menubar-msw.c, redisplay.c, alloc.c, menubar-x.c: Make use of new trapping-errors stuff and rewrite code based on old mechanisms. glyphs-widget.c, redisplay.h: Protect calling Lisp in redisplay. insdel.c: Protect hooks against deleting existing buffers. frame-msw.c: Use EQ, not EQUAL in hash tables whose keys are just numbers. Otherwise we run into stickiness in redisplay because internal_equal() can QUIT. ================================================================ SIGNAL, C-G CHANGES ================================================================ Here we change the way that C-g interacts with event reading. The idea is that a C-g occurring while we're reading a user event should be read as C-g, but elsewhere should be a QUIT. The former code did all sorts of bizarreness -- requiring that no QUIT occurs anywhere in event-reading code (impossible to enforce given the stuff called or Lisp code invoked), and having some weird system involving enqueue/dequeue of a C-g and interaction with Vquit_flag -- and it didn't work. Now, we simply enclose all code where we want C-g read as an event with {begin/end}_dont_check_for_quit(). This completely turns off the mechanism that checks (and may remove or alter) C-g in the read-ahead queues, so we just get the C-g normal. Signal.c documents this very carefully. cmdloop.c: Correct use of dont_check_for_quit to new scheme, remove old out-of-date comments. event-stream.c: Fix C-g handling to actually work. device-x.c: Disable quit checking when err out. signal.c: Cleanup. Add large descriptive comment. process-unix.c, process-nt.c, sysdep.c: Use QUIT instead of REALLY_QUIT. It's not necessary to use REALLY_QUIT and just confuses the issue. lisp.h: Comment quit handlers. ================================================================ CONS CHANGES ================================================================ free_cons() now takes a Lisp_Object not the result of XCONS(). car and cdr have been renamed so that they don't get used directly; go through XCAR(), XCDR() instead. alloc.c, dired.c, editfns.c, emodules.c, fns.c, glyphs-msw.c, glyphs-x.c, glyphs.c, keymap.c, minibuf.c, search.c, eval.c, lread.c, lisp.h: Correct free_cons calling convention: now takes Lisp_Object, not Lisp_Cons chartab.c: Eliminate direct use of ->car, ->cdr, should be black box. callint.c: Rewrote using EXTERNAL_LIST_LOOP to avoid use of Lisp_Cons. ================================================================ USE INTERNAL-BIND-* ================================================================ eval.c: Cleanups of these funs. alloc.c, fileio.c, undo.c, specifier.c, text.c, profile.c, lread.c, redisplay.c, menubar-x.c, macros.c: Rewrote to use internal_bind_int() and internal_bind_lisp_object() in place of whatever varied and cumbersome mechanisms were formerly there. ================================================================ SPECBIND SANITY ================================================================ backtrace.h: - Improved comments backtrace.h, bytecode.c, eval.c: Add new mechanism check_specbind_stack_sanity() for sanity checking code each time the catchlist or specbind stack change. Removed older prototype of same mechanism. ================================================================ MISC ================================================================ lisp.h, insdel.c, window.c, device.c, console.c, buffer.c: Fleshed out authorship. device-msw.c: Correct bad Unicode-ization. print.c: Be more careful when not initialized or in fatal error handling. search.c: Eliminate running_asynch_code, an FSF holdover. alloc.c: Added comments about gc-cons-threshold. dialog-x.c: Use begin_gc_forbidden() around code to build up a widget value tree, like in menubar-x.c. gui.c: Use Qunbound not Qnil as the default for gethash. lisp-disunion.h, lisp-union.h: Added warnings on use of VOID_TO_LISP(). lisp.h: Use ERROR_CHECK_STRUCTURES to turn on ERROR_CHECK_TRAPPING_PROBLEMS and ERROR_CHECK_TYPECHECK lisp.h: Add assert_with_message. lisp.h: Add macros for gcproing entire arrays. (You could do this before but it required manual twiddling the gcpro structure.) lisp.h: Add prototypes for new functions defined elsewhere.
author ben
date Tue, 28 May 2002 08:45:36 +0000
parents 6728e641994e
children 804517e16990
line wrap: on
line diff
--- a/src/events.h	Sat May 25 01:55:30 2002 +0000
+++ b/src/events.h	Tue May 28 08:45:36 2002 +0000
@@ -124,9 +124,9 @@
  unselect_device_cb	(those that use select() and file descriptors and
 			have a separate input fd per device).
 
- create_stream_pair_cb  These callbacks are called by process code to
- delete_stream_pair_cb  create and delete a pair of input and output lstreams
-			which are used for subprocess I/O.
+ create_io_streams_cb   These callbacks are called by process code to
+ delete_io_streams_cb   create the input and output lstreams which are used
+                        for subprocess I/O.
 
  quitp_cb		A handler function called from the `QUIT' macro which
 			should check whether the quit character has been
@@ -160,7 +160,7 @@
   In this case, the handles passed are unix file descriptors, and the code
   may deal with these directly. Although, the same code may be used on Win32
   system with X-Windows. In this case, Win32 process implementation passes
-  handles of type HANDLE, and the create_stream_pair function must call
+  handles of type HANDLE, and the create_io_streams function must call
   appropriate function to get file descriptors given HANDLEs, so that these
   descriptors may be passed to XtAddInput.
 
@@ -203,20 +203,25 @@
   void (*remove_timeout_cb)	(int);
   void (*select_console_cb)	(struct console *);
   void (*unselect_console_cb)	(struct console *);
-  void (*select_process_cb)	(Lisp_Process *);
-  void (*unselect_process_cb)	(Lisp_Process *);
+  void (*select_process_cb)	(Lisp_Process *, int doin, int doerr);
+  void (*unselect_process_cb)	(Lisp_Process *, int doin, int doerr);
   void (*quit_p_cb)		(void);
   void (*force_event_pending)	(struct frame* f);
-  USID (*create_stream_pair_cb) (void* /* inhandle*/, void* /*outhandle*/ ,
+  void (*create_io_streams_cb)  (void* /* inhandle*/, void* /*outhandle*/ ,
+				 void * /* errhandle*/,
 				 Lisp_Object* /* instream */,
 				 Lisp_Object* /* outstream */,
+				 Lisp_Object* /* errstream */,
+				 USID * /* in_usid */, USID * /* err_usid */,
 				 int /* flags */);
-  USID (*delete_stream_pair_cb) (Lisp_Object /* instream */,
-				 Lisp_Object /* outstream */);
+  void (*delete_io_streams_cb)  (Lisp_Object /* instream */,
+				 Lisp_Object /* outstream */,
+				 Lisp_Object /* errstream */,
+				 USID * /* in_usid */, USID * /* err_usid */);
   int (*current_event_timestamp_cb) (struct console *);
 };
 
 /* Flags for create_stream_pair_cb() FLAGS parameter */
 #define STREAM_PTY_FLUSHING		0x0001
 #define STREAM_NETWORK_CONNECTION	0x0002
 
@@ -621,19 +626,28 @@
 /* from event-stream.c */
 Lisp_Object allocate_command_builder (Lisp_Object console, int with_echo_buf);
 void enqueue_magic_eval_event (void (*fun) (Lisp_Object), Lisp_Object object);
-void event_stream_next_event (Lisp_Event *event);
 void event_stream_handle_magic_event (Lisp_Event *event);
 void event_stream_format_magic_event (Lisp_Event *event, Lisp_Object pstream);
 int event_stream_compare_magic_event (Lisp_Event *e1, Lisp_Event *e2);
 Hashcode event_stream_hash_magic_event (Lisp_Event *e);
 void event_stream_select_console   (struct console *con);
 void event_stream_unselect_console (struct console *con);
-void event_stream_select_process   (Lisp_Process *proc);
-void event_stream_unselect_process (Lisp_Process *proc);
-USID event_stream_create_stream_pair (void* inhandle, void* outhandle,
-                Lisp_Object* instream, Lisp_Object* outstream, int flags);
-USID event_stream_delete_stream_pair (Lisp_Object instream, Lisp_Object outstream);
+void event_stream_select_process   (Lisp_Process *proc, int doin, int doerr);
+void event_stream_unselect_process (Lisp_Process *proc, int doin, int doerr);
+void event_stream_create_io_streams (void* inhandle, void* outhandle,
+				     void *errhandle, Lisp_Object* instream,
+				     Lisp_Object* outstream,
+				     Lisp_Object* errstream,
+				     USID* in_usid,
+				     USID* err_usid,
+				     int flags);
+void event_stream_delete_io_streams (Lisp_Object instream,
+				     Lisp_Object outstream,
+				     Lisp_Object errstream,
+				     USID* in_usid,
+				     USID* err_usid);
 void event_stream_quit_p (void);
+void run_pre_idle_hook (void);
 
 struct low_level_timeout
 {
@@ -694,16 +708,25 @@
 
 int event_stream_unixoid_select_console   (struct console *con);
 int event_stream_unixoid_unselect_console (struct console *con);
-int event_stream_unixoid_select_process   (Lisp_Process *proc);
-int event_stream_unixoid_unselect_process (Lisp_Process *proc);
+void event_stream_unixoid_select_process (Lisp_Process *proc, int doin,
+					  int doerr, int *infd, int *errfd);
+void event_stream_unixoid_unselect_process (Lisp_Process *proc, int doin,
+					    int doerr, int *infd, int *errfd);
 int read_event_from_tty_or_stream_desc (Lisp_Event *event,
 					struct console *con);
-USID event_stream_unixoid_create_stream_pair (void* inhandle, void* outhandle,
-                                             Lisp_Object* instream,
-                                             Lisp_Object* outstream,
-                                             int flags);
-USID event_stream_unixoid_delete_stream_pair (Lisp_Object instream,
-                                              Lisp_Object outstream);
+void event_stream_unixoid_create_io_streams (void* inhandle, void* outhandle,
+					     void *errhandle,
+					     Lisp_Object* instream,
+					     Lisp_Object* outstream,
+					     Lisp_Object* errstream,
+					     USID* in_usid,
+					     USID* err_usid,
+					     int flags);
+void event_stream_unixoid_delete_io_streams (Lisp_Object instream,
+					     Lisp_Object outstream,
+					     Lisp_Object errstream,
+					     USID* in_usid,
+					     USID* err_usid);
 
 /* Beware: this evil macro evaluates its arg many times */
 #define FD_TO_USID(fd) ((fd)==0 ? (USID)999999 : ((fd)<0 ? USID_DONTHASH : (USID)(fd)))
@@ -203,20 +203,25 @@
 };
 
-/* Flags for create_stream_pair_cb() FLAGS parameter */
+/* Flags for create_io_streams_cb() FLAGS parameter */
 #define STREAM_PTY_FLUSHING		0x0001
 #define STREAM_NETWORK_CONNECTION	0x0002